Attribute VB_Name = "N6705_functions"
Option Explicit

Dim ioMgr As AgilentRMLib.SRMCls
Public Function set_io(ioaddress As String, answer As String, instrument As VisaComLib.FormattedIO488)
'' This subroutine will set up the VISA-COM IO Library to communicate with an instrument.
'' It will also reset the instrument and check the IDN string.
'' Inputs:
''   ioaddress - the instrument IO address as a string.  Can be LAN, USB, or GPIB
''   instrument - the instrument handle used for the VISA COM library.

    Dim IDNstring As Variant
    On Error GoTo ioError
    
    'The following block sets up all the Instrument IO
    Set ioMgr = New AgilentRMLib.SRMCls
    Set instrument = New VisaComLib.FormattedIO488
    Set instrument.IO = ioMgr.Open(ioaddress)
    instrument.IO.Timeout = 5000
    instrument.FlushRead
    
    'This will make sure that you are communicating properly
    instrument.WriteString "*IDN?"
    answer = instrument.ReadString
    answer = Left(answer, Len(answer) - 1)
    
    'MsgBox "The instrument you are communicating with returns the following *IDN? string:" & vbCrLf & answer
    instrument.WriteString "*RST;*CLS"
    
    set_io = True
    
    Exit Function
ioError:
    MsgBox "Set IO error:" & vbCrLf & Err.Description
    set_io = False
    
End Function

Public Sub close_io(instrument As VisaComLib.FormattedIO488)
    instrument.IO.Close
End Sub


Public Sub set_voltage_current(volts As String, amps As String, chan As String, instrument As VisaComLib.FormattedIO488)
'' This routine will set the voltage and current
'' Inputs
''  volts - the voltage setting as a string
''  amps - the current setting as a string
''  chan - the channel being programmed
''  instrument - the instrument handle used for the VISA COM library.

    Dim chanList As String
    
    chanList = ",(@" & chan & ")"
    
    instrument.WriteString "VOLT " & volts & chanList
    instrument.WriteString "CURR " & amps & chanList
    
    
End Sub

Public Sub set_output_state(state As String, chan As String, instrument As VisaComLib.FormattedIO488)
'' This routine will set the output state.  Send "ON" for on and "OFF" for off.
'' Inputs:
''  state - the desired instrument state.  Musr be "ON" or "OFF".
''  chan - the channel being programmed
''  instrument - the instrument handle used for the VISA COM library.

    Dim chanList As String
    
    chanList = ",(@" & chan & ")"
    
    instrument.WriteString "OUTP " & state & chanList
End Sub

Public Sub set_sine_arb(chan As String, freq As Double, offs As Double, ampl As Double, kind As String, instrument As VisaComLib.FormattedIO488)
'' This routine will setup a sine arb.
'' Inputs:
''  chan - the channel being programmed
''  freq - desired frequency for the sine wave
''  offs - desired offset for the sine wave.  Must be a positive number.
''  ampl - desired amplitude for the sine wave
''  kind - the type of ARB. can be either "VOLT" for voltage or "CURR" for current
''  instrument - the instrument handle used for the VISA COM library.

    Dim chanList, cmdLine As String
    
    chanList = ",(@" & chan & ")"
    
    If kind = "VOLT" Then
        cmdLine = "CURR MAX " & chanList
        instrument.WriteString cmdLine
    Else
        cmdLine = "VOLT MAX " & chanList
        instrument.WriteString cmdLine
    End If
    
    
    cmdLine = "ARB:FUNC:TYPE " & kind & chanList
    instrument.WriteString cmdLine
    cmdLine = "ARB:FUNC:SHAP SIN " & chanList
    instrument.WriteString cmdLine
    cmdLine = kind & ":MODE ARB" & chanList
    instrument.WriteString cmdLine
    
    cmdLine = "ARB:" & kind & ":SIN:OFFS " & CStr(offs) & chanList
    instrument.WriteString cmdLine
    cmdLine = "ARB:" & kind & ":SIN:AMPL " & CStr(ampl) & chanList
    instrument.WriteString cmdLine
    cmdLine = "ARB:" & kind & ":SIN:FREQ " & CStr(freq) & chanList
    instrument.WriteString cmdLine

End Sub

Public Sub enable_arb(chan As String, cont As String, instrument As VisaComLib.FormattedIO488)
''  This routine will enable and initiate an arb
''  Inputs:
''  chan - the channel being programmed
''  cont - intitiate the arb coninuously eith "ON" or "OFF"
''  instrument - the instrument handle used for the VISA COM library.

    Dim chanList, cmdLine As String
    
    chanList = ",(@" & chan & ")"
    cmdLine = "OUTP ON" & chanList
    instrument.WriteString cmdLine
    cmdLine = "INIT:CONT:TRAN " & cont & chanList
    instrument.WriteString cmdLine
    cmdLine = "TRIG:ARB:SOUR IMM"
    instrument.WriteString cmdLine

End Sub

Public Sub setup_elog(chan As String, numparam As Long, trigSour As String, period As Double, doVolt As Boolean, volt_maxmin As Boolean, doCurr As Boolean, curr_maxmin As Boolean, instrument As VisaComLib.FormattedIO488)
'' This routine will setup an external datalog.
'' Inputs
''  chan - the channel being programmed
''  numparam - the number of items being datalogged
''  trigSource - the trigger source, can be "BUS", etc.
''  period - the time for each sampling period of the datalogger
''  doVolt - state of voltage datalog
''  volt_maxmin - state of voltage max/min datalogging
''  doCurr - state of current datalogging
''  curr_maxmin - the state of current max/min datalogging
''  instrument - the instrument handle used for the VISA COM library.
    
    Dim chanList, cmdLine As String
    
    chanList = ",(@" & chan & ")"
    
    instrument.WriteString "SENS:ELOG:FUNC:CURR OFF" & chanList
    
    If doCurr = True Then instrument.WriteString "SENS:ELOG:FUNC:CURR ON" & chanList
    If curr_maxmin = True Then instrument.WriteString "SENS:ELOG:FUNC:CURR:MINM ON" & chanList
    If doVolt = True Then instrument.WriteString "SENS:ELOG:FUNC:VOLT ON" & chanList
    If volt_maxmin = True Then instrument.WriteString "SENS:ELOG:FUNC:VOLT:MINM ON" & chanList

    cmdLine = "SENS:ELOG:PER " & CStr(period) & chanList
    instrument.WriteString cmdLine
    cmdLine = "TRIG:ELOG:SOUR " & trigSour & chanList
    instrument.WriteString cmdLine
End Sub

Public Sub Initiate(chan As String, kind As String, instrument As VisaComLib.FormattedIO488)
''  This routine will initiate either a transient or a triggered measurement
''  Inputs:
''  chan - the channel being programmed
''  kind - what is being initiated, can be "TRAN" or "MEAS"
''  instrument - the instrument handle used for the VISA COM library.

    Dim chanList, cmdLine As String
    
    chanList = " (@" & chan & ")"
    
    cmdLine = "INIT:" & kind & chanList
    instrument.WriteString cmdLine
    
End Sub

Public Sub send_trig(instrument As VisaComLib.FormattedIO488)
''  This routine sends a bus trigger.
''  Inputs:
''    instrument - the instrument handle used for the VISA COM library.

    Dim cmdLine As String
    
    cmdLine = "*TRG"
    instrument.WriteString cmdLine
End Sub

Public Function read_elog(chan As String, instrument As VisaComLib.FormattedIO488)
'' This function will read back an external datalog.
'' Inputs:
''  chan - the channel being programmed
''  instrument - the instrument handle used for the VISA COM library.
'' Output:
''   a varient array of datalogged values

    Dim res_array() As Variant
    Dim cmdLine, chanList As String
    
    chanList = ",(@" & chan & ")"
    
    instrument.FlushRead
    cmdLine = "FETC:ELOG? 4096 " & chanList
    instrument.WriteString cmdLine
    Delay (1)
    res_array = instrument.ReadList
    read_elog = res_array

End Function

Public Sub abort_elog(chan As String, instrument As VisaComLib.FormattedIO488)
'' This routine terminates a datalog
'' Inputs
''  chan - the channel being programmed
''  instrument - the instrument handle used for the VISA COM library.

    Dim cmdLine, chanList As String
    
    chanList = " (@" & chan & ")"
    
    cmdLine = "ABOR:ELOG" & chanList
    instrument.WriteString cmdLine
    
End Sub

Public Function get_numb_chan(instrument As VisaComLib.FormattedIO488)
'' This function returns the number of power supply modules installed in a mainframe.
'' Input:
''  instrument - the instrument handle used for the VISA COM library.
'' Output
''  An interger representing the number of channels installed

    instrument.WriteString "SYST:CHAN?"
    get_numb_chan = instrument.ReadString

End Function

Public Function get_chan_mod(chan As String, instrument As VisaComLib.FormattedIO488)
'' This functions will return the model number of a channel installed in a mainframe
'' Inputs:
''  chan - the channel being programmed
''  instrument - the instrument handle used for the VISA COM library.
'' Output:
''  the model number as a string

    Dim chanList As String
    
    chanList = " (@" & chan & ")"
    
    instrument.WriteString "SYST:CHAN:MOD? " & chanList
    get_chan_mod = instrument.ReadString
    
End Function

Public Function chkModSettings(voltSet As Double, currSet As Double, modelN As String)
'' This function will check the voltage and current settings of a module to ensure they are within limits
'' Inputs:
''  voltSet - the set voltage
'   currSet - the set current
'   modelN - the model number of the module being programmed
'' Output:
'   1 if all settings are good and 0 if it fails

    Dim maxVolt As Double
    Dim maxCurr As Double
    
    Select Case modelN
        Case "N6731B"
            maxVolt = 5.1
            maxCurr = 10.2
        Case "N6732B"
            maxVolt = 8.16
            maxCurr = 6.375
        Case "N6733B"
            maxVolt = 20.4
            maxCurr = 2.55
        Case "N6734B"
            maxVolt = 35.7
            maxCurr = 1.53
        Case "N6735B"
            maxVolt = 61.2
            maxCurr = 0.85
        Case "N6736B"
            maxVolt = 102
            maxCurr = 0.51
        Case "N6741B"
            maxVolt = 5.1
            maxCurr = 20.4
        Case "N6732B"
            maxVolt = 8.16
            maxCurr = 12.75
        Case "N6733B"
            maxVolt = 20.4
            maxCurr = 5.1
        Case "N6734B"
            maxVolt = 35.7
            maxCurr = 3.06
        Case "N6735B"
            maxVolt = 61.2
            maxCurr = 1.7
        Case "N6736B"
            maxVolt = 102
            maxCurr = 1.02
        Case "N6751A"
            maxVolt = 51
            maxCurr = 5.1
        Case "N6752A"
            maxVolt = 51
            maxCurr = 10.2
        Case "N6753A"
            maxVolt = 20.4
            maxCurr = 51
        Case "N6754A"
            maxVolt = 61.2
            maxCurr = 20.4
        Case "N6761A"
            maxVolt = 51
            maxCurr = 1.53
        Case "N6762A"
            maxVolt = 51
            maxCurr = 3.06
        Case "N6773A"
            maxVolt = 20.4
            maxCurr = 15.3
        Case "N6774A"
            maxVolt = 35.7
            maxCurr = 8.67
        Case "N6775A"
            maxVolt = 61.2
            maxCurr = 5.1
        Case "N6776A"
            maxVolt = 102
            maxCurr = 3.06
    End Select
    
    If currSet > maxCurr Then
        MsgBox "The set current, " & CStr(currSet) & "A exceeds the maximum current setting of " & CStr(maxCurr) & "A! Please choose a current setting within the limits."
        chkModSettings = 0
        Exit Function
    ElseIf currSet < 0 Then
        MsgBox "This module can not be set to less than 0 A!  Please enter a value that is greater than zero."
        chkModSettings = 0
        Exit Function
    End If

    If voltSet > maxVolt Then
        MsgBox "The set voltage, " & CStr(currSet) & "V exceeds the maximum votlage setting of " & CStr(maxVolt) & "V! Please choose a voltage setting within the limits."
        chkModSettings = 0
        Exit Function
    ElseIf voltSet < 0 Then
        MsgBox "This module can not be set to less than 0 V!  Please enter a value that is greater than zero."
        chkModSettings = 0
        Exit Function
    End If
    
    chkModSettings = 1

End Function

Public Function chkSampRate(rate As Double, numSamp As Long)
'' This function checks if the sample rate for the datalog is within limits
'' Inputs:
''  rate - the sampling rate
''  numSamp - the number of samples
'' Output:
'   1 if all settings are good and 0 if it fails

    Dim maxRate As Double
    Dim totalmin As Double
    
    maxRate = 0.00010248
    totalmin = maxRate * numSamp
    
    If rate < totalmin Then
        MsgBox "The minimum sample period is " & CStr(totalmin) & "s.  Please reset."
        chkSampRate = 0
        Exit Function
    End If
    
    chkSampRate = 1
End Function
